/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

/**
 * @file globus_gsi_authz_error.c
 * @brief Globus GSI Authz Library
 * @author Sam Meder
 */

#include "globus_gsi_authz_constants.h"

char * 
globus_l_gsi_authz_error_strings[GLOBUS_GSI_AUTHZ_ERROR_LAST] =
{
/* 0 */   "Success",
/* 1 */   "Error with system call",
/* 2 */   "Invalid parameter",
/* 3 */   "Callout returned an error"
};

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
